@@ -1,8 +1,6 @@
-import sys
 import logging
-from subprocess import CalledProcessError
 from typing import List, Tuple
-from pycparser import parse_file, c_ast
+from pycparser import c_ast
 from pycparser.c_ast import Node, Assignment, If, While, For, Compound
 
 from .relation_list import RelationList, Relation
@@ -19,46 +17,34 @@ class Analysis:
 
     @staticmethod
     def run(
-            file_in: str, file_out: str = None, no_save: bool = False,
-            use_cpp: bool = True, cpp_path: str = None, cpp_args: str = None
+            ast: c_ast, file_out: str = None, no_save: bool = False
     ) -> Tuple[Relation, List[List[int]]]:
         """Run MWP analysis on specified input file.
 
         Arguments:
-            file_in: C source code file path
+            ast: parsed C source code AST
             file_out: where to store result
             no_save: Set true when analysis result should not be saved to file
-            use_cpp: Set to True if you want to execute the C pre-processor
-                on the file prior to parsing it.
-            cpp_path: If use_cpp is True, this is the path to 'cpp' on your
-                system. If no path is provided, it attempts to just execute
-                'cpp', so it must be in your PATH.
-            cpp_args: If use_cpp is True, set this to the command line
-                arguments strings to cpp. Be careful with quotes - it's best
-                to pass a raw string (r'') here. If several arguments are
-                required, pass a list of strings.
 
         Returns:
               Computed relation and list of non-infinity choices.
-
         """
 
         choices = [0, 1, 2]
-        logger.info(f'Starting analysis of {file_in}')
-        ast = Analysis.parse_c_file(file_in, use_cpp, cpp_path, cpp_args)
-        Analysis.validate_ast(ast)
 
         function_body = ast.ext[0].body
-        index, relations, combinations = 0, RelationList(), []
+        index, combinations = 0, []
+        variables = Analysis.find_variables(function_body)
+        relations = RelationList.identity(variables=variables)
         total = len(function_body.block_items)
         delta_infty = False
         dg = DeltaGraph()
 
         for i, node in enumerate(function_body.block_items):
             logger.debug(f'computing relation...{i} of {total}')
-            index, rel_list, exit_ = Analysis.compute_relation(index, node, dg)
-            if exit_:
-                delta_infty = True
+            index, rel_list, delta_infty = Analysis \
+                .compute_relation(index, node, dg)
+            if delta_infty:
                 break
             logger.debug(f'computing composition...{i} of {total}')
             relations.composition(rel_list)
@@ -71,18 +57,55 @@ class Analysis:
             save_relation(file_out, relations.first, combinations)
 
         logger.debug(f'\nMATRIX{relations}')
-        if not combinations and relations.first.variables:
+
+        if delta_infty:
             logger.info('infinite')
-            # Should not raise here since delta_graph takes care of it
+
+        # Should not raise here since delta_graph takes care of it
+        # conditional on: (1) that some variable(s) exist, so we do not
+        # conclude infinity on simple programs and
+        # (2) regarding index > see: issue #43
+        elif not combinations and relations.first.variables and index > 0:
+            logger.info('infinite undetected by delta graph')
             assert False
-            # We can get empty combinations for simple programs
-            # => also check that some variables exist, so we do not raise
-            # infinity on simple programs
         else:
             logger.info(f'CHOICES:\n{combinations}')
 
         return relations.first, combinations
 
+    @staticmethod
+    def find_variables(function_body: Compound) -> List[str]:
+        """Finds all local variable declarations in function body.
+
+        This method scans recursively AST nodes looking for
+        variable declarations. For each declaration, the
+        name of the variable will be recorded. Method returns
+        a list of all discovered variable names.
+
+        Arguments:
+            function_body: AST node with sub-nodes.
+
+        Returns:
+            List of all discovered variable names, or
+            empty list if no variables were found.
+        """
+        variables = []
+
+        def recurse_nodes(node_):
+            # only look for declarations
+            if isinstance(node_, c_ast.Decl):
+                variables.append(node_.name)
+            if hasattr(node_, 'block_items'):
+                for sub_node in node_.block_items:
+                    recurse_nodes(sub_node)
+
+        # search for declarations
+        if hasattr(function_body, 'block_items'):
+            for node in function_body.block_items:
+                recurse_nodes(node)
+
+        return variables
+
     @staticmethod
     def compute_relation(index: int, node: Node, dg: DeltaGraph) \
             -> Tuple[int, RelationList, bool]:
@@ -101,12 +124,14 @@ class Analysis:
 
         logger.debug("in compute_relation")
 
+        if isinstance(node, c_ast.Decl):
+            return index, RelationList(), False
         if isinstance(node, c_ast.Assignment):
             if isinstance(node.rvalue, c_ast.BinaryOp):
                 index, rel_list = Analysis.binary_op(index, node)
                 return index, rel_list, False
             if isinstance(node.rvalue, c_ast.Constant):
-                index, rel_list = Analysis.constant(index, node)
+                index, rel_list = Analysis.constant(index, node.lvalue.name)
                 return index, rel_list, False
             if isinstance(node.rvalue, c_ast.UnaryOp):
                 index, rel_list = Analysis.unary_op(index, node)
@@ -210,8 +235,9 @@ class Analysis:
         return index, rel_list
 
     @staticmethod
-    def constant(index: int, node: Assignment) -> Tuple[int, RelationList]:
-        """Analyze a constant.
+    def constant(index: int, variable_name: str) -> Tuple[int, RelationList]:
+        """Analyze a constant assignment of form: x = c where x is some
+        variable and c is constant.
 
         From MWP paper:
 
@@ -221,15 +247,13 @@ class Analysis:
 
         Arguments:
             index: delta index
-            node: node representing a constant
+            variable_name: name of variable to which constant is being assigned
 
         Returns:
             Updated index value and relation list
         """
-
-        logger.debug('Computing Relation (second case / constant)')
-        var_name = node.lvalue.name
-        return index, RelationList([var_name])
+        logger.debug('Constant value node')
+        return index, RelationList([variable_name])
 
     @staticmethod
     def unary_op(index: int, node: Assignment) -> Tuple[int, RelationList]:
@@ -496,61 +520,3 @@ class Analysis:
                  for val, scalar in enumerate(p2_scalars)]))
 
         return index + 1, vector
-
-    @staticmethod
-    def parse_c_file(
-            file: str, use_cpp: bool, cpp_path: str, cpp_args: str
-    ) -> c_ast:
-        """Parse C file using pycparser.
-
-        Arguments:
-            file: path to C file
-            use_cpp: Set to True if you want to execute the C pre-processor
-                on the file prior to parsing it.
-            cpp_path: If use_cpp is True, this is the path to 'cpp' on your
-                system. If no path is provided, it attempts to just execute
-                'cpp', so it must be in your PATH.
-            cpp_args: If use_cpp is True, set this to the command line
-                arguments strings to cpp. Be careful with quotes - it's best
-                to pass a raw string (r'') here. If several arguments are
-                required, pass a list of strings.
-
-        Returns:
-            Generated AST
-        """
-        try:
-            ast = parse_file(file, use_cpp, cpp_path, cpp_args)
-            if use_cpp:
-                info = f'parsed with preprocessor: {cpp_path} {cpp_args}'
-            else:
-                info = 'parsed without preprocessor'
-            logger.debug(info)
-            return ast
-        except CalledProcessError:
-            logger.error('Failed to parse C file. Terminating.')
-            sys.exit(1)
-
-    @staticmethod
-    def validate_ast(ast: c_ast) -> None:
-        """Check if successfully parsed AST can be analyzed.
-
-        Here we check that the C input file contains some source code
-        (has body) and that that body is not empty (has block_items).
-        These types of inputs do not cause the parser to error, so we
-        need to check these separately from parse error.
-
-        If the input is invalid terminate immediately.
-
-        Ref: [issue #4](https://github.com/seiller/pymwp/issues/4)
-
-        Arguments:
-            ast: AST object
-        """
-
-        invalid = ast is None or ast.ext is None or len(ast.ext) == 0
-        invalid = invalid or ast.ext[0].body is None or ast.ext[
-            0].body.block_items is None
-
-        if invalid:
-            logger.error('Input C file is invalid or empty.')
-            sys.exit(1)
